--1) for each month, how many products have been purchased in the month by at
--least 30 distinct customers each?

with member nprods as 
count(filter([Product].[Product Id].[Product Id], [Measures].[NCustomers] >=20))
select nprods on columns,
nonempty(([Time].[The Month].[The Month])) on rows
from sales

--2) for each store, and considering only Saturdays, the products that have sold at
--least 1% of sales of the store;

with member totsales as 
([Product].[Product Id].currentmember.parent, [Measures].[Store Sales])
member percsales as [Measures].[Store Sales]/totsales
select {[Measures].[Store Sales], totsales} on columns,
nonempty( 
Filter( ([Store].[Store Id].[Store Id],[Product].[Product Id].[Product Id]), percsales>0.01) ) on rows
from sales
where [Time].[WeekDays].[The Day].&[Saturday]

-- 3) for every product category, percentage of customers buying that product category
--on Tuesday's over the total number of customers buying that category;

with member perc as  
([Time].[WeekDays].[The Day].&[Tuesday], [Measures].[NCustomers])  / [Measures].[NCustomers],
FORMAT_STRING="Percent" 
select perc on columns,
[Product].[Product Category].[Product Category] on rows 
from sales

-- 4) For every customer city, number of different product categories bought by customers of that city
with member ncat as  
Count( nonempty( DISTINCT([Product].[Product Category].[Product Category], [Measures].[Store Sales]) ) ) 
select ncat on columns,
[Customer].[City].[City] on rows 
from sales 

-- 5) for every product category and customer country, the store with the highest 
-- percentage of sales over the total sales of that category and country. Result must be ordered by
-- percentage descending.

WITH member total as ([Store].[Store Id].[All], [Measures].[Store Sales]) 
MEMBER perc AS [Measures].[Store Sales]/total,
FORMAT_STRING="Percent" 
SELECT perc on columns, 
Order(
 {Generate( nonempty(([Product].[Product Category].[Product Category], [Customer].[Country].[Country])),
  topcount( ([Product].[Product Category].currentmember,[Customer].[Country].currentmember,
             [Store].[Store Id].[Store Id]), 1, perc))},
 perc, bdesc) on rows
FROM sales

-- Sol. Analytic SQL
with rk as 
(select pc.product_category, c.country, s.store_id, 
         sum(store_sales)/sum(sum(store_sales)) over (partition by pc.product_category, c.country) as ratio,
         rank() over (partition by pc.product_category, c.country order by sum(store_sales) desc) as rk
from sales_fact s join product p on s.product_id = p.product_id
       join product_class pc on p.product_class_id = pc.product_class_id
       join customer c on s.customer_id = c.customer_id 
group by pc.product_category, c.country, s.store_id) 

select product_category, country, store_id, ratio
from rk where rk=1
order by ratio desc

--6) for every product, the date with the highest sales of the product. If there are
--two or more days with the same highest sales, pick up any date.

select [Measures].[Store Sales]  on columns,
generate( [Product].[Product Id].[Product Id],
 topcount( ([Product].[Product Id].currentmember, [Time].[Time Id].[Time Id]), 
 1, [Measures].[Store Sales]) ) on rows
from sales

-- Analytic SQL
with temp as (
select product_id, time_id, sum(store_sales) as sales, 
rank() over (partition by product_id order by sum(store_sales), time_id desc) as rk
from sales_fact
group by product_id, time_id)

select product_id, time_id, sales
from temp
where rk=1

-- 20140205
with member supp as [Measures].[Sales Fact Count]
member suppA as ([Store].[Geography].[All], [Measures].[Sales Fact Count])
member conf as supp/suppA

select {supp, suppA, conf} on columns,
filter( (descendants([Product].[Marketing]),descendants([Store].[Geography])), conf>0.1) on rows
from sales

select [Measures].[Sales Fact Count] on columns,
descendants([Product].[Marketing] on rows
from sales
 


